/***********************************************************************
 *               Copyright (C) 1995 Joe English
 *                   Freely redistributable
 ***********************************************************************
 *
 * File:	stream.c
 * Author: 	Joe English
 * Created:	25 Feb 1995
 * Description: Routines to manage ESISStreams
 *
 * $Date: 96/05/24 17:10:04 $
 * $Revision: 1.6 $
 * $Locker:  $
 */

#include <stdlib.h>
#include <stdio.h>
#include "project.h"
#include "strmap.h"
#include "pile.h"
#include "esis.h"
#include "esisp.h"

ESISStream estream_create(ESISIOproc buffill, void *bufclosure)
{
    ESISStream ep;

    ep = malloc(sizeof(*ep));
    if (!ep) return NULL;
    ep->rdptr = ep->rootnode = ep->curnode = ep->wrptr = 0;
    ep->relations = 0;
    ep->datapile = pcreate();
    ep->pathno = 1;

    ep->rootnode = esis_open_node(ep,EN_SD);
    ep->rootnode->depth = 0;
    ep->rootnode->pathno = -1;
    ep->rdptr = ep->rootnode;
    ep->curnode = ep->rootnode;
    ep->wrptr = 0;

    ep->bufsize = BUFSIZ;
    ep->buf = malloc(ep->bufsize);
    if (!ep->buf) { estream_close(ep); return 0; }
    ep->bufp = ep->buf;
    ep->bufcnt = 0;
    ep->buffill = buffill;
    ep->bufclosure = bufclosure;

    return ep;
}

void estream_close(ESISStream ep)
{
    pdestroy(ep->datapile);
    /* %%% free nodes ... */
    /* %%% free buffer ... */
    free(ep);
}

/*+++ Inquiry routines: 
*/
ESISNode estream_rootnode(ESISStream ep)
{
    return ep->rootnode;
}


int estream_load(ESISStream ep)
{
    ESISNode evnode;
    ESISEventType eventType;

    do {
	eventType = estream_next(ep, &evnode);
    } while (eventType != EV_EOF && eventType != EV_ERROR);

    return eventType == EV_EOF;
}

/*+++ I/O proc for stdio:
*/
int ESISIOstdio(void *closure, char *buf, int n)
{
     FILE *fp = (FILE *)closure;
     return fread(buf, sizeof(char), n, fp);
}
