/* 
 * File: tclAppInit.c
 * Tcl_AppInit() and main() routines for Cost.
 * This file is used for both costsh (if -DUSE_TK is not defined)
 * and for costwish (if it is.)
 *
 * Got a favorite Tcl extension that I've left out?  Let me know...
 *
 * NB: Not all combinations of extensions have been tested.
 *
 * Joe English <joe@art.com>
 */

#ifndef USE_ITCL	/* require: version 2.0 */
#define USE_ITCL 0
#endif
#ifndef USE_TCLX	/* require: version ??? */
#define USE_TCLX 0
#endif
#ifndef USE_TK		/* require: version 4.0 for Tcl 7.4, 4.1 for Tcl 7.5 */
#define USE_TK 0
#endif
#ifndef USE_BLT		/* require: version > 1.8 */
#define USE_BLT 0
#endif
#ifndef USE_TIX		/* require: version ??? */
#define USE_TIX 0
#endif

#include <stdlib.h>
#include "tcl.h"

/* Extension headers: */
#if USE_ITCL
#	include "itcl.h"
#endif
#if USE_TK
#	include "tk.h"
#endif
#if USE_TCLX
#	include "tclExtend.h"
#endif

/* Declarations for extensions w/no headers:  */

extern int Cost_Init(Tcl_Interp *);

extern int Blt_Init(Tcl_Interp *);	/* #if USE_BLT */
extern int Tix_Init(Tcl_Interp *);	/* #if USE_TIX */

#define EXTENSION(interp, name, pkg_Init) \
	if (pkg_Init(interp) != TCL_OK) return TCL_ERROR; \
	Tcl_SetVar2(interp, "COST", "EXTENSIONS", name, \
		TCL_GLOBAL_ONLY | TCL_LIST_ELEMENT | TCL_APPEND_VALUE);

int Tcl_AppInit(Tcl_Interp *interp)
{
    if (Tcl_Init(interp) != TCL_OK)
	return TCL_ERROR;

#if USE_TCLX
    EXTENSION(interp, "TCL", TclX_Init)
#endif
#if USE_ITCL
    EXTENSION(interp, "ITCL", Itcl_Init)
#endif 
#if USE_TK
    EXTENSION(interp, "TK", Tk_Init);
#	if USE_BLT
		EXTENSION(interp, "BLT", Blt_Init);
#	endif
#	if USE_TIX
		EXTENSION(interp, "TIX", Tix_Init);
#	endif
#endif	/* USE_TK */


    EXTENSION(interp, "COST", Cost_Init);


#if !USE_TK
    /* If we're in batch mode (sgmls ... | costsh ...) 
     * and 'cost_commandline' fails, there's a good chance
     * that stdin has not yet been consumed.  Since we
     * _definitely_ don't want to feed sgmls' output to
     * Tcl, we exit() instead of returning an error.
     */
    if (Tcl_Eval(interp, "cost_commandline") != TCL_OK) 
	exit(1);	
#endif

    return TCL_OK;
}

int main(int argc, char *argv[])
{
#if USE_TK
    Tk_Main(argc, argv, Tcl_AppInit);
#else
    Tcl_Main(argc, argv, Tcl_AppInit);
#endif
    return 0;
}
